:- module(combinator, [replace_comb/3, make_combinator/3]).

:- use_module(library(lists), [difference/3, sublist/2, append/3]).
:- use_module(curryast,       [get_variables/2, split_rule/3, get_params/2, get_function/2]).

:- include(ast_ops).

%% Utilities.

%% When doing lambda lifting, sometimes we have to convert a function
%% into a super combinator, thus we add some parameters:

%%    f a = let g b    = h a b  in g a a ->
%%    f a = let g a' b = h a' b in g a a

%% replace_func replaces an expr id by other.
replace_comb(FName@Args, rinfo(FName, NewVars), FName@NArgs) :-
	!,
	%% If Args already contains the Vars fail.
        (
	    sublist(NewVars, Args) ->
	    fail
	;
	    append(NewVars, Args, NArgs)
	).

make_combinator(Rule, NewRule, rinfo(FName, Closure)) :-
	split_rule(Rule, Head, Body),
	get_variables(Head, PVar),
	get_variables(Body, BVar),
	uniq(PVar, PVars),
	uniq(BVar, BVars),
	difference(BVars, PVars, Closure),
	get_function(Head, FName),
	get_params(Head, HeadP),
	append(Closure, HeadP, NewHeadP),
	split_rule(NewRule, FName@NewHeadP, Body).
	    
	

%% Returns a set from a list.
uniq(L, RL) :-
	(
	    setof(X, member(X, L), RL),
	    !
	;
	    RL = []
	).

	
%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

