:- module(prolog2curry, [prolog_to_curry/2, prolog_to_type/2]).

%% This module is a pretty printer for prolog expressions.

:- use_module(library(terms), [atom_concat/2]).

:- include(type_ops).

%% Debemos usar vndict.

pretty_name(A, B) :-
	var(A),
	!,
	prettyvars(A),
	A = B.
pretty_name(A, B) :-
	number(A),
	!, 
	A = B.

%% Should be done right with the typer.
%%pretty_name
pretty_name(A, B) :-
	list(A),
	!,
	A = B.

pretty_name(A, B) :-
	atom_codes(A, Sr),
	append(Mod, [0'_|Name], Sr), 
	atom_codes(ModA, Mod),
	atom_codes(NameA, Name),
	atom_concat([ModA, '.', NameA], B).

prolog_to_curry(A, B) :-
	pretty_name(A, B).

pretty_type(A, A) :-
	var(A),
	prettyvars(A),
	!.

pretty_type(T --> T1, R) :-
	pretty_type(T, TN),
	pretty_type(T1, T1N),
	atom_concat([TN, '->', T1N], R).

pretty_type(T<@Args, R) :-
	pretty_type(T, TN),
	pretty_type_list(Args, ArgsN),
	atom_concat([TN,' '|ArgsN], R).

pretty_type(tid(Mod, Name), B) :-
	atom_concat([Mod, '.', Name], B).

pretty_type_list([], []).
pretty_type_list([T|TL], R) :-
	pretty_type_list(TL, TN),
	pretty_type(T, TT),
	(
	    TN = [] ->
	    R = TT
	;
	    atom_concat([TT,' '|TN], R)
	).

prolog_to_type(A, B) :-
	pretty_type(A, B).
