:- module(trans_cons, [translate_constructors/1]).

:- use_module(symtab,    [get_constructors/2, get_object_property/3, object_arity/2] ).
:- use_module(curryast,  [make_name/2] ).
:- use_module(types,     [type_name/2] ).

:- use_module(library(lists), [append/3, list_concat/2]).

translate_constructors(Prolog) :-
	get_module_constructors(Cons), 
	!,
	is_in_hnf_constructors(Cons, Cs),
	!,
	constructors_to_hnf(Cons, Rs),                       
 	!,
	constructors_to_nf(Cons, Rnfs),
	!,
	append(Cs, Rs, CRs),
	append(CRs, Rnfs, Prolog).
	

%% constructors To hnf
constructors_to_hnf([], []).
constructors_to_hnf([C|CL], [R|RL]) :-
	object_arity(C, Arity),
	cons_tohnf(C, Arity, R),
	constructors_to_hnf(CL, RL).

cons_tohnf(Name, Arity, Res) :-
	new_vars(Arity, Vars),
	make_name(Name, SName),
	NCons =.. [SName|Vars],
	Res = (user_tohnf(NCons, NCons) :- !).

%% cons to nf.
constructors_to_nf([], []).
constructors_to_nf([C|CL], [R|RL]) :-
	object_arity(C, Arity),
	cons_tonf(C, Arity, R),
	constructors_to_nf(CL, RL).

cons_tonf(Name, Arity, Res) :-
	new_vars(Arity, Vars1),
	copy_term(Vars1, Vars2),
	cons_tonf_list(Vars1, Vars2, Body),
	make_name(Name, SName),
	NCons1 =.. [SName|Vars1],
	NCons2 =.. [SName|Vars2],
	Res = (user_tonf(NCons1, NCons2) :- Body).

cons_tonf_list([], [], (!)).
cons_tonf_list([L1|LL1], [L2|LL2], (Tonf1, TonfL)) :-
	Tonf1 = tonf(L1, L2),
	!,
	cons_tonf_list(LL1, LL2, TonfL).


%% Outputs the is_constructor/1 form.
  
is_in_hnf_constructors([], []).
is_in_hnf_constructors([C|CL], [R|RL]) :-
	hnf_cons(C, R),
	is_in_hnf_constructors(CL, RL).

hnf_cons(Cons, is_constructor(SCons)) :-
	make_name(Cons, SCons).

%% Aux 

new_vars(0, []) :- !.
new_vars(N, [_|Ts]) :-
	N1 is N-1,
	new_vars(N1, Ts).

%% Should avoid imported constructors, so checks that the type name
%% belongs to this module.
get_module_constructors(List) :-
	get_object_property(ModName, exports, _),
	findall(ConsList, (
			      get_constructors(Type, ConsList),
			      type_name(Type, tid(ModName, _))
			  ), CCList),
	list_concat(CCList, List).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

