:- module(types, [check_type/1, type_arity/2, type_name/2,
                  map_type/3, map_types/1, fake_type/2]).

:- use_package(hiord).

:- use_module(symtab, [get_object_property/3, replace_object_property/3]).

% TODO: Should export these ops to people who import this module.
:- include(type_ops).

%% Kill this early, please ;(
fake_type(0, tvar(a)).
fake_type(N, tvar(a) --> B) :-
	N1 is N - 1,
	fake_type(N1, B).


%% Calc the type arity. Used extensibly on the translator.
type_arity(X, 0) :-
	var(X),
	!.

type_arity(_A --> B, N) :-
	!,
	type_arity(B, N1),
	N is N1 + 1.

type_arity(_, 0) :-
	!.

%% Return qid of a type.
type_name(Name<@_Args, Name).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% A map predicate for types. 

:- meta_predicate map_types(pred(2)).

map_types(Func) :-
	get_object_property(Name, type, Type),
	map_type(Type, Func, NType),
	replace_object_property(Name, type, NType),
	fail.
map_types(_F).

:- meta_predicate map_type(?, pred(2), ?).
map_type(T, Func, NT) :- 
	Func(T, T1),
	!,
	map_type(T1, Func, NT). % See map_tree

map_type(A, _F, A) :-
	var(A),
	!.

map_type(tid(M, N), _F, tid(M, N)) :-
	!.
map_type(tvar(A), _F, tvar(A)) :-
	!.

map_type(T1 --> T2, F, NT1 --> NT2) :-
	map_type(T1, F, NT1),
	map_type(T2, F, NT2),
	!.
map_type(T<@TL, F, NT<@NTL) :-
	map_type(T, F, NT),
	!,
	map_type_list(TL, F, NTL).

:- meta_predicate map_type_list(?, pred(2), ?).

map_type_list([], _F, []) :-
	!.
map_type_list([T|TL], Func, [NT|NTL]) :-
	map_type(T, Func, NT),
	!,
	map_type_list(TL, Func, NTL).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check if Term is a valid type.
%% An atom is a type name.
check_type(tid(_, _)).

check_type(tv(_)).

check_type(X<@TVars) :-
	check_type(X),
	check_type_list(TVars).

check_type(A-->B) :-
	check_type(A),
	check_type(B).

check_type((_TClass=>T)) :-
	check_type(T).

check_type_list([]).
check_type_list([T|TL]) :-
	check_type(T),
	check_type_list(TL).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:
